/*
 * Parameter modifier for BJC-600J `ESC (' and `ESC [' commands.
 *
 * Usage: paramod cmd nth param...
 *
 * e.g. paramod c 0 0x10 -1 0x01 modifies all occurences of
 * 		ESC ( c 03 00 ll mm nn
 * to		ESC ( c 03 00 10 mm 01
 * if `nth' parameter is other than 0, only the nth occurence of the command
 * will be modified.
 *
 * Create: 940618
 */

#include <stdio.h>

printusage() {
  fprintf(stderr,
" Parameter modifier for BJC-600J `ESC (' and `ESC [' commands.\n\n"
" Usage: paramod cmd nth param...\n\n"
" e.g. paramod c 0 0x10 -1 0x01 modifies all occurences of\n"
"        ESC ( c 03 00 ll mm nn\n"
"   to   ESC ( c 03 00 10 mm 01\n"
" if `nth' parameter is other than 0, only the nth occurence of the command\n"
" will be modified.\n"
	  );
  exit(1);
}

main (argc, argv)
int argc;
char *argv [];
{
  int i, n, nth, cmd, del, v;
  int c1, c2, c3, c4, c5, prefix;

  if (argc < 3) {
    printusage();
  }

  if (argc == 3) del = 1; else del = 0;
  cmd = argv[1][0];
  nth = strtol(argv[2], 0, 0);
  argv += 3;
  argc -= 3;

  n = 0;
  while ((c1 = getchar()) != EOF) {
    if (c1 == 033) {
      if ((c2 = getchar()) == '(' || c2 == '[') {
	if ((c3 = getchar()) == cmd && (nth == ++n || !nth)) {
	  prefix=c2;
	  c1 = getchar();
	  c2 = getchar();
	  if (del) for (i=0; i < c1+c2*256; i++) getchar();
	  else {
	    putchar(033); putchar(prefix); putchar(cmd);
	    putchar(c1); putchar(c2);
	    for (i=0; i < argc; i++) {
	      v = strtol(argv[i], 0, 0);
	      c3 = getchar();
	      putchar(v < 0 ? c3 : v);
	    }
	    for (; i < c1+c2*256; i++) putchar(getchar());
	  }
	  
	  continue;
	}
      } else {putchar(c1); putchar(c2); continue;}
    } else {putchar(c1); continue;}
    putchar(c1); putchar(c2); putchar(c3);
    c1 = putchar(getchar());
    c2 = putchar(getchar());
    for (i=0; i < c1+c2*256; i++) putchar(getchar());
  }

  if (nth) fprintf(stderr, "%dth occurence of ESC %c %c is ", nth, prefix, cmd);
  else fprintf(stderr, "%d occurence(s) of ESC %c %c are ", n, prefix, cmd);
  if (del) {
    fprintf(stderr, "deleted.\n");
  } else {
    fprintf(stderr,
	    "modified to \n"
	    "\tESC %c %c n1 n2",
	    prefix, cmd);
    for (i=0; i < argc; i++) {
      v = strtol(argv[i], 0, 0);
      if (v < 0) fprintf(stderr, " nn");
      else fprintf(stderr, " %2x", v);
    }
    putc('\n', stderr);
  }
}
